@@ -7,6 +7,7 @@ Contact: dse.ssd@gmail.com
 """
 import os
 import sys
+import io
 import zipfile
 import glob
 from pathlib import Path
@@ -139,6 +140,14 @@ def list_files_with_extension(
         name_exclude_filter=None, name_include_filter=None):
     """List files that have a specific extension."""
 
+    if isinstance(ext, list):
+        file_list = []
+        for ext1 in ext:
+            file_list += list_files_with_extension(
+                path=path, ext=ext1, name_exclude_filter=name_exclude_filter,
+                name_include_filter=name_include_filter)
+        return file_list
+
     if ext[0] == '.':
         print("Specify extension as 'txt', do not include the dot")
 
@@ -441,11 +450,12 @@ def read_ezlog(file_name):
     return log
 
 
-def read_beam_steering_log():
+def read_beam_steering_log(dir_list=None):
     """Read T4 beam steering positioning log."""
     signal_names = [
         'A Motor X', 'A Motor Y', 'B Motor X', 'B Motor Y',
-        'A Measured X', 'A Measured Y', 'B Measured X', 'B Measured Y']
+        'A Measured X', 'A Measured Y', 'B Measured X', 'B Measured Y',
+        'A FeedbackState', 'B FeedbackState', 'A Total Power', 'B Total Power']
     pos_log = [dict() for signal in signal_names]
 
     for ind, signal in enumerate(signal_names):
@@ -453,67 +463,83 @@ def read_beam_steering_log():
         pos_log[ind]['val'] = np.array([])
         pos_log[ind]['signal_names'] = signal_names
 
-    for dir_name_month in list_dirs('positioning'):
-        for dir_name_day in list_dirs('positioning/' + dir_name_month):
-            for ind, signal in enumerate(signal_names):
-                file_name = 'positioning/' + dir_name_month + '/' + \
-                    dir_name_day + '/{:s}.txt'.format(signal)
-                print("Reading file ", file_name)
-                try:
-                    pos_log_data = np.loadtxt(
-                        file_name, delimiter=',', usecols=[0, 2],
-                        dtype={'names': ('date', 'pos'),
-                               'formats': (datetime.datetime, 'float')})
+    if not dir_list:
+        for dir_name_month in list_dirs('positioning'):
+            for dir_name_day in list_dirs('positioning/' + dir_name_month):
+                dir_list = 'positioning/' + dir_name_month + '/' + dir_name_day
+
+    for dir in dir_list:
+        for ind, signal in enumerate(signal_names):
+            file_name = dir + '/{:s}.txt'.format(signal)
+            print("Reading file ", file_name)
+            try:
+                if Path(Path(file_name).parts[0]).suffix == '.zip':
+                    zip_file_name = Path(Path(file_name).parts[0])
+                    archive = zipfile.ZipFile(zip_file_name, mode='r')
+                    file = io.BufferedReader(archive.open(Path(*Path(file_name).parts[1:]).as_posix(), mode='r'))
+                else:
+                    file = open(file_name)
+
+                pos_log_data = np.loadtxt(
+                    file, delimiter=',', usecols=[0, 2],
+                    dtype={'names': ('date', 'pos'),
+                            'formats': (datetime.datetime, 'float')})
+
+                if pos_log[ind].get('t0') is None:
+                    pos_log[ind]['t0'] = datetime.datetime.strptime(
+                        pos_log_data[0][0], "%Y-%m-%d %H:%M:%S.%f")
+
+                pos_log[ind]['tarr'] = np.append(
+                    pos_log[ind]['tarr'],
+                    np.array([(datetime.datetime.strptime(entry[0],
+                                "%Y-%m-%d %H:%M:%S.%f")
+                                - pos_log[ind]['t0']).total_seconds()/60/60
+                                for entry in pos_log_data]))
+
+                pos_log[ind]['val'] = np.append(
+                    pos_log[ind]['val'],
+                    np.array([entry[1] for entry in pos_log_data]))
+
+            except ValueError as excpt:
+                print("Failed to read log file with exception", excpt)
+                print("Retrying line-by-line")
+                if Path(Path(file_name).parts[0]).suffix == '.zip':
+                    file = io.BufferedReader(archive.open(Path(*Path(file_name).parts[1:]).as_posix(), mode='r'))
+                else:
+                    file = open(file_name)
+
+                for line in file:
+                    if isinstance(line, bytes):
+                        line = line.decode('utf-8')
+                    col_data = line.split(',')
+
+                    # Make sure there are exacly three columns in the line
+                    if len(col_data) != 3:
+                        continue
+
+                    # Parse the timestamp and do some sanity checks
+                    try:
+                        data_ts = datetime.datetime.strptime(
+                            col_data[0], "%Y-%m-%d %H:%M:%S.%f")
+                        if data_ts.year < 1990 or data_ts.year > 2100:
+                            continue
+
+                        data_val = float(col_data[2])
+                        if np.abs(data_val) > 1000:
+                            continue
+                    except Exception:
+                        continue
 
                     if pos_log[ind].get('t0') is None:
-                        pos_log[ind]['t0'] = datetime.datetime.strptime(
-                            pos_log_data[0][0], "%Y-%m-%d %H:%M:%S.%f")
+                        pos_log[ind]['t0'] = data_ts
 
                     pos_log[ind]['tarr'] = np.append(
                         pos_log[ind]['tarr'],
-                        np.array([(datetime.datetime.strptime(entry[0],
-                                   "%Y-%m-%d %H:%M:%S.%f")
-                                   - pos_log[ind]['t0']).total_seconds()/60/60
-                                  for entry in pos_log_data]))
+                        float((data_ts -
+                                pos_log[ind]['t0']).total_seconds()/60/60))
 
                     pos_log[ind]['val'] = np.append(
                         pos_log[ind]['val'],
-                        np.array([entry[1] for entry in pos_log_data]))
-
-                except Exception as excpt:
-                    print("Failed to read log file with exception", excpt)
-                    print("Retrying line-by-line")
-                    file = open(file_name)
-                    for line in file:
-                        col_data = line.split(',')
-
-                        # Make sure there are exacly three columns in the line
-                        if len(col_data) != 3:
-                            continue
-
-                        # Parse the timestamp and do some sanity checks
-                        try:
-                            data_ts = datetime.datetime.strptime(
-                                col_data[0], "%Y-%m-%d %H:%M:%S.%f")
-                            if data_ts.year < 1990 or data_ts.year > 2100:
-                                continue
-
-                            data_val = float(col_data[2])
-                            if np.abs(data_val) > 1000:
-                                continue
-                        except Exception:
-                            continue
-
-                        if pos_log[ind].get('t0') is None:
-                            pos_log[ind]['t0'] = data_ts
-
-                        pos_log[ind]['tarr'] = np.append(
-                            pos_log[ind]['tarr'],
-                            float((data_ts -
-                                   pos_log[ind]['t0']).total_seconds()/60/60))
-
-                        pos_log[ind]['val'] = np.append(
-                            pos_log[ind]['val'],
-                            data_val)
+                        data_val)
 
     return pos_log
