@@ -24,6 +24,7 @@ from lkcom.util import isnone, get_color, bin_and_slice_data, hex2byte, \
     get_granularity, handle_general_exception, hist_bin_centers_to_edges, \
     extend_range, printmsg
 from lkcom.string import get_human_val_str, rem_extension, strip_whitespace
+from lkcom.standard_func import gaussian_1D
 
 
 def get_pt2mm():
@@ -176,7 +177,7 @@ def show_png_ext(FileName):
 def hist(
         yarr, yrng=None, target_num_bins=256, hist_bin_sz_fac=1, ygran=None,
         with_histogram_dither=False, hist_dith_factor=4, hide_x_labels=False,
-        orientation="vertical", ymarkers=None, ylim=None,
+        orientation="vertical", xmarkers=None, ymarkers=None, ylim=None, log=False,
         color=matplotlib.cm.get_cmap('Blues')(0.95)):
     """Plot a nice histogram with dithering and data granularity awareness."""
     if yrng is None:
@@ -190,7 +191,7 @@ def hist(
     print('Y granularity is {:.2e}, scale range is {:.2f} bits'.format(
         ygran, np.log2(yspan/ygran)))
 
-    unique_vals = np.unique(yarr)
+    unique_vals = np.unique(yarr).astype('int32')
 
     if len(unique_vals) < target_num_bins:
         # If the number of unique data values is less than the number of
@@ -226,14 +227,21 @@ def hist(
         dith = 1
 
     plt.hist(yarr*dith, bins=bins, orientation=orientation,
-             color=color, zorder=10, log=False)
+             color=color, zorder=10, log=log, alpha=0.5)
     ax = plt.gca()
 
+    if xmarkers:
+        for xmarker in xmarkers:
+            add_x_marker(xmarker, c=[0, 0, 0], ls='-')
+
     if ymarkers:
         # Add histogram y markers
         for ymarker in ymarkers:
             add_y_marker(ymarker, c=[0.75, 0.75, 0.75], ls='-', zorder=1)
 
+    plt.xlim([bins[0], bins[-1]])
+    plt.gca().set_ylim(bottom=1)
+
     plt.ylim(ylim)
     plt.plot(plt.xlim(), [0, 0], color=[0, 0, 0, 0.25])
     if hide_x_labels:
@@ -246,6 +254,66 @@ def hist(
     return ax
 
 
+def make_histogram_panel(varr, ax_hist=None, ylim=None, rsd_val=None,
+        ax_trace=None, hist_orientation='horizontal',
+        log=True):
+    """Make a histogram panel.
+
+    Data in varr is plotted as histogram in a panel. The histogram can be
+    displayed horizontally or vertically and fits nicely as a side panel.
+    """
+    if not ax_hist:
+        ax_hist = plt.gca()
+
+    if not ylim:
+        ylim = [0, 1]
+
+    plt.sca(ax_hist)
+    hist_rng = ylim
+
+    y_span = hist_rng[1] - hist_rng[0]
+    y_gran = get_granularity(varr)
+    num_hist_bins = y_span/y_gran
+    target_hist_bins = 500
+    y_grans_per_bin = np.ceil(num_hist_bins/target_hist_bins)
+    bins = np.arange(hist_rng[0], hist_rng[1], y_gran*y_grans_per_bin)
+    if(abs(len(bins)/target_hist_bins - 1) > 0.1):
+        print("Number of histogram bins requested: {:d}, actual number "
+              "is {:d} due to Y data granularity".format(
+                target_hist_bins, len(bins)))
+
+    hist_y = plt.hist(varr, bins=bins, log=log, alpha=1.0, orientation=hist_orientation)[0]
+
+    plt.grid('on')
+    set_ticks_inside()
+
+    if rsd_val:
+        gauss_x = np.linspace(hist_rng[0], hist_rng[1], 500)
+        gauss_y = gaussian_1D(gauss_x, A=np.max(hist_y), sigma=rsd_val, c=1)
+        if hist_orientation == 'horizontal':
+            if log:
+                plt.semilogx(gauss_y, gauss_x, c='k', alpha=0.75)
+            else:
+                plt.plot(gauss_y, gauss_x, c='k', alpha=0.75)
+        else:
+            plt.plot(gauss_x, gauss_y, c='k', alpha=0.75)
+
+    if hist_orientation == 'horizontal':
+        plt.xlim([1, np.max(hist_y)*1.1])
+        ax_hist.xaxis.set_ticks([])
+        ax_hist.yaxis.set_ticklabels([])
+        if ax_trace:
+            ax_hist.set_yticks(ax_trace.get_yticks())
+        plt.ylim(ylim)
+        # add_infobox('log hist.') # does not work with log scale...
+    else:
+        ax_hist.yaxis.set_ticks([])
+        plt.xlabel('Energy, a.u.')
+        align_outer_tick_labels(ax_hist)
+
+
+
+
 def CompareHistograms(
         img_arr, names=None, range=None, bins=64, histtype='step', xlabel=None,
         ylabel='Occurence', title=None):
@@ -264,7 +332,9 @@ no_signal_color = [0.25, 0.25, 0.25]
 
 
 def imshow_ex(
-        img, min_vspan=None, vmin=None, vmax=None, ax=None,
+        img, ax=None,
+        vmin=None, vmax=None, vrng_algorithm='minmax', min_vspan=None,
+        auto_vrng_percentile_min=0, auto_vrng_percentile_max=99.5,
         bad_color=no_signal_color, logscale=False, cmap='virids',
         title_str=None, with_hist=True, is_angle=False):
     """Show an image with a histogram and colorbar.
@@ -286,11 +356,21 @@ def imshow_ex(
         img2[invmask] = np.log10(img[invmask])
         img = img2
 
+    if vrng_algorithm not in ['minmax', 'perctentile']:
+        print("Unknown vrng algorithm, resetting to 'minmax")
+        vrng_algorithm = 'minmax'
+
     if vmin is None:
-        vmin = np.nanmin(img)
+        if vrng_algorithm == 'minmax':
+            vmin = np.nanmin(img)
+        elif vrng_algorithm == 'percentile':
+            vmin = np.nanpercentile(img.flatten(), auto_vrng_percentile_min)
 
     if vmax is None:
-        vmax = np.nanmax(img)
+        if vrng_algorithm == 'minmax':
+            vmax = np.nanmax(img)
+        elif vrng_algorithm == 'percentile':
+            vmax = np.nanpercentile(img.flatten(), auto_vrng_percentile_max)
 
     if min_vspan is not None and vmax - vmin < min_vspan:
         vmean = np.mean([vmin, vmax])
@@ -348,6 +428,46 @@ def imshow_ex(
         tick_labels = [get_human_val_str(10**x) for x in ticks]
         cbar_ax.set_xticklabels(tick_labels)
 
+def add_infobox(
+        infobox_str=None, show_infobox=True, infobox_hpos='left', text_va='top',
+        infobox_vpos='top', ref_y=None, text_x=None, text_y=None, **kwargs):
+    if show_infobox and infobox_str:
+        xl = kwargs.get('xlim')
+        if xl is None:
+            xl = plt.gca().get_xlim()
+
+        yl = kwargs.get('ylim')
+        if yl is None:
+            yl = plt.gca().get_ylim()
+
+        ofs_x = (xl[1] - xl[0])*0.05
+        ofs_y = (yl[1] - yl[0])*0.07
+
+        text_ha = 'left'
+        if not text_x:
+            if infobox_hpos == 'left':
+                text_x = xl[0] + ofs_x
+                text_ha = 'left'
+            elif infobox_hpos == 'right':
+                text_x = xl[1] - ofs_x
+                text_ha = 'right'
+
+        if not text_y:
+            if infobox_vpos == 'top':
+                if ref_y is None:
+                    ref_y = yl[1]
+                text_y = ref_y - ofs_y
+            elif infobox_vpos == 'middle':
+                text_y = (yl[1] - yl[0])/2
+            elif infobox_vpos == 'bottom':
+                if ref_y is None:
+                    ref_y = yl[0]
+                text_y = ref_y + ofs_y
+
+        plt.text(text_x, text_y, infobox_str, ha=text_ha, va=text_va,
+                 bbox=dict(facecolor='white', alpha=0.75,
+                 edgecolor='white'))
+
 
 def add_marker(
         pos=None, label=None, axis='x', c='k', ls='--', zorder=None, xlim=None,
